@@ -1,4 +1,5 @@
-#$Id: Entity.pm 276 2013-11-09 23:45:30Z maj $
+#$Id: Entity.pm 285 2013-11-16 17:41:28Z maj $
+use v5.10;
 package REST::Neo4p::Entity;
 use REST::Neo4p::Exceptions;
 use Carp qw(croak carp);
@@ -9,7 +10,7 @@ use warnings;
 
 # base class for nodes, relationships, indexes...
 BEGIN {
-  $REST::Neo4p::Entity::VERSION = '0.2001';
+  $REST::Neo4p::Entity::VERSION = '0.2200';
 }
 
 our $ENTITY_TABLE = {};
@@ -27,13 +28,14 @@ sub new {
   }
   my ($properties) = (@_);
   my $url_components = delete $properties->{_addl_components};
-  my $agent = $REST::Neo4p::AGENT;
+  my $agent = REST::Neo4p->agent;
   REST::Neo4p::CommException->throw("Not connected\n") unless $agent;
   my $decoded_resp;
   eval {
-    $decoded_resp = $agent->post_data([$entity_type,
-					$url_components ? @$url_components : ()],
-				       $properties);
+    $decoded_resp = $agent->post_data(
+      [$entity_type, $url_components ? @$url_components : ()],
+      $properties
+     );
   };
   if (my $e = REST::Neo4p::Exception->caught()) {
     # TODO : handle cases
@@ -71,29 +73,32 @@ sub new_from_json_response {
   my $self_url  = $decoded_resp->{self} || $decoded_resp->{template};
   $self_url =~ s/{key}.*$//; # another kludge for get_indexes
   my ($obj) = $self_url =~ /([a-z0-9_]+)\/?$/i;
+  my $tbl_entry = $ENTITY_TABLE->{$entity_type}{$obj};
   my ($start_id,$end_id);
   if ($decoded_resp->{start}) {
     ($start_id) = $decoded_resp->{start} =~ /([0-9]+)\/?$/;
     ($end_id) = $decoded_resp->{end} =~ /([0-9]+)\/?$/;
   }
-  unless (defined $ENTITY_TABLE->{$entity_type}{$obj}) {
+  unless (defined $tbl_entry) {
     if ($decoded_resp->{template}) {     # another kludge for get_indexes
       ($decoded_resp->{type}) = $decoded_resp->{template} =~ m|index/([a-z]+)/|;
     }
-    $ENTITY_TABLE->{$entity_type}{$obj}{entity_type} = $entity_type;
-    $ENTITY_TABLE->{$entity_type}{$obj}{self} = bless \$obj, $class;
-    $ENTITY_TABLE->{$entity_type}{$obj}{self_url} = $self_url;
-    $ENTITY_TABLE->{$entity_type}{$obj}{start_id} = $start_id;
-    $ENTITY_TABLE->{$entity_type}{$obj}{end_id} = $end_id;
-    $ENTITY_TABLE->{$entity_type}{$obj}{batch} = 0;
-    $ENTITY_TABLE->{$entity_type}{$obj}{type} = $decoded_resp->{type};
+    $tbl_entry = $ENTITY_TABLE->{$entity_type}{$obj} = {};
+    $tbl_entry->{entity_type} = $entity_type;
+    $tbl_entry->{self} = bless \$obj, $class;
+    $tbl_entry->{self_url} = $self_url;
+    $tbl_entry->{start_id} = $start_id;
+    $tbl_entry->{end_id} = $end_id;
+    $tbl_entry->{batch} = 0;
+    $tbl_entry->{type} = $decoded_resp->{type};
+    $tbl_entry->{_handle} = REST::Neo4p->handle; # current db handle
   }
   if ($REST::Neo4p::CREATE_AUTO_ACCESSORS && ($entity_type ne 'index')) {
-    my $self =  $ENTITY_TABLE->{$entity_type}{$obj}{self};
+    my $self =  $tbl_entry->{self};
     my $props = $self->get_properties;
     for (keys %$props) { $self->_create_accessors($_) unless $self->can($_); }
   }
-  return $ENTITY_TABLE->{$entity_type}{$obj}{self};
+  return $tbl_entry->{self};
 }
 
 sub new_from_batch_response {
@@ -104,12 +109,14 @@ sub new_from_batch_response {
     REST::Neo4p::NotSuppException->throw("Cannot use ".__PACKAGE__." directly\n");
   }
   my ($id_token) = (@_);
-  $ENTITY_TABLE->{$entity_type}{$id_token}{entity_type} = $entity_type;
-  $ENTITY_TABLE->{$entity_type}{$id_token}{self} = bless \$id_token, $class;
-  $ENTITY_TABLE->{$entity_type}{$id_token}{self_url} = $id_token;
-  $ENTITY_TABLE->{$entity_type}{$id_token}{batch} = 1;
-  $ENTITY_TABLE->{batch_objs}->{$id_token} =  $ENTITY_TABLE->{$entity_type}{$id_token}{self};
-  return $ENTITY_TABLE->{$entity_type}{$id_token}{self};
+  my $tbl_entry = $ENTITY_TABLE->{$entity_type}{$id_token} = {};
+  $tbl_entry->{entity_type} = $entity_type;
+  $tbl_entry->{self} = bless \$id_token, $class;
+  $tbl_entry->{self_url} = $id_token;
+  $tbl_entry->{_handle} = REST::Neo4p->handle; # current handle
+  $tbl_entry->{batch} = 1;
+  $ENTITY_TABLE->{batch_objs}{$id_token} = $tbl_entry->{self};
+  return $tbl_entry->{self};
 }
 
 # remove() - delete the node and destroy the object
@@ -118,7 +125,9 @@ sub remove {
   my @url_components = @_;
   my $entity_type = ref $self;
   $entity_type =~ s/.*::(.*)/\L$1\E/;
-  my $agent = $REST::Neo4p::AGENT;
+  local $REST::Neo4p::HANDLE;
+  REST::Neo4p->set_handle($self->_handle);
+  my $agent = REST::Neo4p->agent;
   eval {
     $agent->delete_data($entity_type, @url_components, $$self);
   };
@@ -139,7 +148,9 @@ sub set_property {
   REST::Neo4p::LocalException->throw("Arg must be a hashref\n") unless ref($props) && ref $props eq 'HASH';
   my $entity_type = ref $self;
   $entity_type =~ s/.*::(.*)/\L$1\E/;
-  my $agent = $REST::Neo4p::AGENT;
+  local $REST::Neo4p::HANDLE;
+  REST::Neo4p->set_handle($self->_handle);
+  my $agent = REST::Neo4p->agent;
   my $suffix = $self->_get_url_suffix('property');
   my @ret;
   $suffix =~ s|/[^/]*$||; # strip the '{key}' placeholder
@@ -170,7 +181,9 @@ sub get_property {
   my @props = @_;
   my $entity_type = ref $self;
   $entity_type =~ s/.*::(.*)/\L$1\E/;
-  my $agent = $REST::Neo4p::AGENT;
+  local $REST::Neo4p::HANDLE;
+  REST::Neo4p->set_handle($self->_handle);
+  my $agent = REST::Neo4p->agent;
   REST::Neo4p::CommException->throw("Not connected\n") unless $agent;
   my $suffix = $self->_get_url_suffix('property');
   my @ret;
@@ -200,7 +213,9 @@ sub get_properties {
   my $self = shift;
   my $entity_type = ref $self;
   $entity_type =~ s/.*::(.*)/\L$1\E/;
-  my $agent = $REST::Neo4p::AGENT;
+  local $REST::Neo4p::HANDLE;
+  REST::Neo4p->set_handle($self->_handle);
+  my $agent = REST::Neo4p->agent;
   REST::Neo4p::CommException->throw("Not connected\n") unless $agent;
   my $suffix = $self->_get_url_suffix('property');
   $suffix =~ s|/[^/]*$||; # strip the '{key}' placeholder
@@ -243,7 +258,9 @@ sub remove_property {
   my @props = @_;
   my $entity_type = ref $self;
   $entity_type =~ s/.*::(.*)/\L$1\E/;
-  my $agent = $REST::Neo4p::AGENT;
+  local $REST::Neo4p::HANDLE;
+  REST::Neo4p->set_handle($self->_handle);
+  my $agent = REST::Neo4p->agent;
   REST::Neo4p::CommException->throw("Not connected\n") unless $agent;
   my $suffix = $self->_get_url_suffix('property');
   foreach (@props) {
@@ -298,7 +315,7 @@ sub _entity_by_id {
   }
   unless ($ENTITY_TABLE->{$entity_type}{$id}) {
     # not recorded as object yet
-    my $agent = $REST::Neo4p::AGENT;
+    my $agent = REST::Neo4p->agent;
     REST::Neo4p::CommException->throw("Not connected\n") unless $agent;
     my ($rq, $decoded_resp);
     if ($entity_type eq 'index') {
@@ -356,13 +373,6 @@ sub _get_url_suffix {
   my $suffix = $ENTITY_TABLE->{$entity_type}{_actions}{$action};
 }
 
-sub _self_url {
-  my $self = shift;
-  my $entity_type = ref $self;
-  $entity_type =~ s/.*::(.*)/\L$1\E/;
-  return $ENTITY_TABLE->{$entity_type}{$$self}{self_url};
-}
-
 # get the $ENTITY_TABLE entry for the object
 sub _entry {
   my $self = shift;
@@ -370,6 +380,18 @@ sub _entry {
   $entity_type =~ s/.*::(.*)/\L$1\E/;
   return $ENTITY_TABLE->{$entity_type}{$$self};
 }
+sub _self_url {
+  my $self = shift;
+  return $self->_entry->{self_url} if $self->_entry;
+  return;
+}
+
+# get the $ENTITY_TABLE entry for the object
+sub _handle { 
+  my $self = shift;
+  return $self->_entry->{_handle} if $self->_entry;
+  return;
+}
 
 sub DESTROY {
   my $self = shift;
