@@ -1,11 +1,11 @@
-#$Id: Path.pm 41 2012-11-22 01:14:10Z maj $
+#$Id: Path.pm 276 2013-11-09 23:45:30Z maj $
 package REST::Neo4p::Path;
 use REST::Neo4p::Exceptions;
 use Carp qw(croak carp);
 use strict;
 use warnings;
 BEGIN {
-  $REST::Neo4p::Path::VERSION = '0.2001';
+  $REST::Neo4p::Path::VERSION = '0.2120';
 }
 
 sub new {
@@ -54,6 +54,25 @@ sub new_from_json_response {
   return $obj;
 }
 
+sub as_simple {
+  my $self = shift;
+  my $ret;
+  my @n = $self->nodes;
+  my @r = $self->relationships;
+  while (my $n = shift @n) {
+    push @$ret, $n->as_simple;
+    my $r = shift @r;
+    push @$ret, $r->as_simple if defined $r;
+  }
+  return $ret;
+}
+
+sub simple_from_json_response {
+  my $class = shift;
+  my ($decoded_resp) = @_;
+  return $class->new_from_json_response($decoded_resp)->as_simple;
+}
+
 sub nodes { @{shift->{_nodes}} }
 sub relationships { @{shift->{_relationships}} }
 
@@ -100,6 +119,16 @@ Get the nodes in path order.
 
 Get the relationships in path order.
 
+=item as_simple()
+
+ $a = $path->as_simple;
+ @simple_nodes = grep { $_->{_node} } @$a;
+ @simple_relns = grep { $_->{_relationship} } @$a;
+
+Get the path as an array of simple node and relationship hashes (see
+L<REST::Neo4p::Node/as_simple()>,
+L<REST::Neo4p::Relationship/as_simple()>).
+
 =back
 
 =head1 SEE ALSO
@@ -115,7 +144,7 @@ L<REST::Neo4p::Query>.
 
 =head1 LICENSE
 
-Copyright (c) 2012 Mark A. Jensen. This program is free software; you
+Copyright (c) 2012-2013 Mark A. Jensen. This program is free software; you
 can redistribute it and/or modify it under the same terms as Perl
 itself.
 
